# Generated by Django 2.1.3 on 2018-11-28 16:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("fcm_django", "0003_auto_20170313_1314"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="fcmdevice",
            options={
                "verbose_name": "FCM device",
                "verbose_name_plural": "FCM devices",
            },
        ),
    ]
