# Generated by Django 1.10.6 on 2017-03-13 13:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fcm_django", "0002_auto_20160808_1645"),
    ]

    operations = [
        migrations.AlterField(
            model_name="fcmdevice",
            name="type",
            field=models.CharField(
                choices=[("ios", "ios"), ("android", "android"), ("web", "web")],
                max_length=10,
            ),
        ),
    ]
